/* ************************************************************************** *
 *  Filename: netif.c
 *  Functions: 	init_connection(), send_message(), receive_message(),
 *	        close_connection().
 *
 *  This client program is developed based on a sample client.
 *  Client Program (Network Protocol Routines).
 *  by Neo Say Poh/ Eldwin Tan Say Lin.
 * ************************************************************************** */

#include "Agent.h"


/* ************************************************************************** *
 * init_connection() - Open a UDP/IP socket.
 *
 * Args:
 * 		host : host name 	(Default: localhost) 
 * 		port : port number	(Default: 6000) 
 * Return type:
 * 		Socket : socket assigned
 * ************************************************************************** */
Socket init_connection(char *host, int port)
{
    	struct hostent 		*host_ent ;
    	struct in_addr      	*addr_ptr ;
    	struct sockaddr_in	cli_addr ;
    	int			sockfd, val ;
    	Socket			sock ;

    	sock.socketfd = -1 ;

    	if((host_ent = (struct hostent *)gethostbyname(host)) == NULL) 
    	{
		/* Check if a numeric address */
		if(inet_addr(host) == -1)
		{
		    	return sock ;
		}
    	}
    	else
    	{
		addr_ptr = (struct in_addr *) *host_ent->h_addr_list ;
		host = inet_ntoa(*addr_ptr) ;
    	}

     	/*  Open UDP socket.  */
    	if( (sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    	{
		return sock ;  	/* Can't open socket. */
    	}

    	val = fcntl(sockfd, F_GETFL, 0) ;
    	val |= O_NONBLOCK ;
    	fcntl(sockfd, F_SETFL, val) ;

     	/*  Bind any local address.  */
    	bzero((char *) &cli_addr, sizeof(cli_addr)) ;
    	cli_addr.sin_family		= AF_INET ;
    	cli_addr.sin_addr.s_addr	= htonl(INADDR_ANY) ;
    	cli_addr.sin_port		= htons(0) ;


    	if(bind(sockfd, (struct sockaddr *) &cli_addr, sizeof(cli_addr)) < 0)
    	{
		return sock ;  /* Can't bind local address */
    	}
    

     	/*  Fill in the structure with the address of the server.  */
    	sock.socketfd = sockfd ;

    	bzero((char *) &sock.serv_addr, sizeof(sock.serv_addr)) ;
    	sock.serv_addr.sin_family		= AF_INET ;
    	sock.serv_addr.sin_addr.s_addr	= inet_addr(host) ;
    	sock.serv_addr.sin_port		= htons(port) ;

    	return sock ;
}


/* ************************************************************************** *
 * send_message() - Send a message to server.
 *
 * Args:
 *		sock : socket
 *		buf  : message
 * Return type:
 * 		Bool : 	TRUE  - Successful
 *	       	   	FALSE - Failed
 * ************************************************************************** */
Bool send_message(Socket sock, char *buf)
{
    	int n ;

    	n = strlen(buf) ;
/*    
    	printf( "send %d : %s\n",
	ntohs(sock.serv_addr.sin_port),buf);
*/
    	if (sendto(sock.socketfd, buf, n, 0,
	    	(struct sockaddr *)&sock.serv_addr, sizeof(sock.serv_addr)) != n)
		return FALSE ;
    	else
		return TRUE ;
}

	   
/* ************************************************************************** *
 * receive_message() - Receive a message from the server.
 *
 * Args:
 * 		sock : socket
 * 		buf  : message 
 * 		size : sizeof(buf)
 * Return type:
 *		int  :  1 - Successful.
 *	       	   	0 - No message. 
 *	      	  -1 - Failed.
 * ************************************************************************** */
int receive_message(Socket *sock, char *buf, int size)
{
    	int			n,servlen ;
    	struct sockaddr_in	serv_addr ;
    
    	servlen = sizeof(serv_addr) ;
    	n = recvfrom(sock->socketfd, buf, size, 0,
		 (struct sockaddr *)&serv_addr, &servlen);

    	if(n < 0)
	{
		if((n == -1) && (errno == EWOULDBLOCK))
		{
    			buf[0] = '\0';
    			return 0 ;
		}
		else
	    		return (-1) ;
	}
    	else
    	{
/*	
		printf( "recv %d : ", ntohs(serv_addr.sin_port));
*/
		sock->serv_addr.sin_port = serv_addr.sin_port ;
		buf[n] = '\0' ;

		if(n == 0)
		    	return 0 ;
		else
		    	return 1 ;
    	}
}


/* ************************************************************************** *
 * close_connection() - Close UDP/IP socket.
 *
 * Arg: 
 * 		sock : socket.
 * Return type:
 * 		void.
 * ************************************************************************** */
void close_connection(Socket sock)
{
    	close(sock.socketfd) ;
}
